<?php

namespace GCFormsApprovals\Admin;

/**
 * Class FormBuilder 构建表单生成器内部的功能
 *
 * @since 1.0.0
 */
class FormBuilder {

	/**
	 * White list of field types to allow for mapping select.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	protected $allowed_field_types = [];

	/**
	 * Initialize.
	 *
	 * @since 1.0.0
	 */
	public function init() {

		$this->hooks();
	}

	/**
	 * Hooks.
	 *
	 * @since 1.0.0
	 */
	protected function hooks() {

		add_action( 'gcforms_form_settings_panel_content', [ $this, 'panel_content' ],   40 );
		add_action( 'gcforms_builder_enqueues',            [ $this, 'enqueue_assets' ],  10 );

		add_filter( 'gcforms_builder_settings_sections',   [ $this, 'panel_sidebar' ],   40, 2 );
		add_filter( 'gcforms_builder_strings',             [ $this, 'builder_strings' ], 40, 2 );
		add_filter( 'gcforms_get_form_fields_allowed',     [ $this, 'form_fields_allowed' ] );
	}

	/**
	 * 为审批流管理面板添加内容
	 *
	 * @since 1.0.0
	 *
	 * @param \GCForms_Builder_Panel_Settings $builder_panel_settings GCForms_Builder_Panel_Settings object.
	 */
	public function panel_content( $builder_panel_settings ) {

		gcforms_approvals()->settings->set_props( $builder_panel_settings );
		$approvals = gcforms_approvals()->settings->get_prop( 'approvals' );

		echo gcforms_render( // phpcs:ignore GeChiUI.Security.EscapeOutput.OutputNotEscaped
			GCFORMS_APPROVALS_PATH . 'views/settings/section',
			[
				'next_id'             => max( array_keys( $approvals ) ) + 1,
				'enable_control_html' => $this->get_enable_control_html(),
				'approvals_html'       => $this->get_approvals_html(),
				'add_new_btn_classes' => $this->get_html_class(
					[
						'gcforms-builder-settings-block-add',
						'gcforms-approvals-add',
					]
				),
			]
		);
	}

	/**
	 * 审批流功能“开启/关闭”
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	protected function get_enable_control_html() {

		return gcforms_panel_field(
			'select',
			'settings',
			'approvals_enable',
			gcforms_approvals()->settings->get_prop( 'form_data' ),
			esc_html__( '审批', 'gcforms-approvals' ),
			[
				'default' => '0',
				'options' => [
					'1' => esc_html__( '开启', 'gcforms-approvals' ),
					'0' => esc_html__( '关', 'gcforms-approvals' ),
				],
			],
			false
		);
	}

	/**
	 * 审批流HTML
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	protected function get_approvals_html() {

		$approvals = gcforms_approvals()->settings->get_prop( 'approvals' );
		$result   = '';

		foreach ( $approvals as $approval_id => $approval ) {
			$approval['approval_id'] = $approval_id;

			$result .= $this->get_approval_block( $approval );
		}

		return $result;
	}

	/**
	 * 审批流区块框架HTML
	 *
	 * @since 1.0.0
	 *
	 * @param array $approval Approval data.
	 *
	 * @return string
	 */
	protected function get_approval_block( $approval ) {

		$approval_id   = $approval['approval_id'];
		$form_data    = gcforms_approvals()->settings->get_prop( 'form_data' );
		$toggle_state = '<i class="fa fa-chevron-up"></i>';
		$closed_state = '';

		if (
			! empty( $form_data['id'] ) &&
			'closed' === gcforms_builder_settings_block_get_state( $form_data['id'], $approval_id, 'approval' )
		) {
			$toggle_state = '<i class="fa fa-chevron-down"></i>';
			$closed_state = 'style="display:none;"';
		}

		$result = gcforms_render(
			GCFORMS_APPROVALS_PATH . 'views/settings/block',
			[
				'id'            => $approval_id,
				'name'          => $approval['name'],
				'toggle_state'  => $toggle_state,
				'closed_state'  => $closed_state,
				'fields'        => $this->get_approval_fields( $approval ),
				'block_classes' => $this->get_html_class(
					[
						'gcforms-builder-settings-block',
						'gcforms-builder-settings-block-approval',
					]
				),
			]
		);

		/**
		 * Filter a HTML for setting block.
		 *
		 * @since 1.0.0
		 *
		 * @param string $result     HTML for setting block.
		 * @param array  $form_data  Form data.
		 * @param int    $approval_id Approval ID.
		 */
		return apply_filters( 'gcforms_approvals_form_builder_get_approval_block', $result, $form_data, $approval_id );
	}

	/**
	 * 审批流区块中的HTML操作字段
	 *
	 * @since 1.0.0
	 *
	 * @param array $approval Approval data.
	 *
	 * @return string
	 */
	protected function get_approval_fields( $approval ) {

		$approval_id    = $approval['approval_id'];
		$approvals      = gcforms_approvals()->settings->get_prop( 'approvals' );
		$form_data     = gcforms_approvals()->settings->get_prop( 'form_data' );
		$allowed_types = implode( ' ', $this->allowed_field_types );

		$result = gcforms_panel_field(
			'select',
			'approvals',
			'user_id',
			$form_data,
			esc_html__( '审批人', 'gcforms-approvals' ),
			[
				'parent'      => 'settings',
				'subsection'  => $approval_id,
				'default'     => 'json',
				'options'    => gcforms_approvals()->get_approver_users(),
				// 'placeholder' => esc_html__( '输入审批人...', 'gcforms-approvals' ),
				'tooltip'     => esc_html__( '请选择审批人', 'gcforms-approvals' ),
			],
			false
		);

		$result .= gcforms_panel_field(
			'select',
			'approvals',
			'order',
			$form_data,
			esc_html__( '审批排序', 'gcforms-approvals' ),
			[
				'parent'      => 'settings',
				'subsection'  => $approval_id,
				'input_id'    => 'gcforms-panel-field-approvals-order-' . $approval_id,
				'default'     => '1',
				'options'    => gcforms_approvals()->get_available_orders(),
				'tooltip'     => esc_html__( '支持顺序审批（Lv1-10排序），同时支持并行审批（选择同一个级别）', 'gcforms-approvals' ),
			],
			false
		);

		$result .= gcforms_render(
			GCFORMS_APPROVALS_PATH . 'views/settings/status-mapping',
			[
				'title'         => esc_html__( '自定义审批状态', 'gcforms-approvals' ),
				'approval_id'    => $approval_id,
				// 'fields'        => $form_fields,
				'allowed_types' => $allowed_types,
				'meta'          => ! empty( $approvals[ $approval_id ]['status'] ) ? $approvals[ $approval_id ]['status'] : [ false ],
				'name'          => "settings[approvals][{$approval_id}][status]",
			]
		);


		// 条件逻辑是Pro版本功能
		if (
			! function_exists( 'gcforms' ) ||
			! gcforms()->pro ||
			version_compare( gcforms()->version, '1.6.1.2', '<' )
		) {
			$result .= '<div class="gcforms-status-map-table">自动审批功能需GCforms Pro版本支持</div>';

		}else{
			$result .= gcforms_conditional_logic()->builder_block(
				[
					'form'        => $form_data,
					'type'        => 'panel',
					'panel'       => 'approvals',
					'parent'      => 'settings',
					'subsection'  => $approval_id,
					'actions'     => [
						'confirm'   => esc_html__( '自动通过', 'gcforms-approvals' ),
						'reject'   => esc_html__( '自动驳回', 'gcforms-approvals' )
					],
					'action_desc' => esc_html__( ' 当下列条件满足', 'gcforms-approvals' ),
				],
				false
			);
		}

		/**
		 * Filter HTML for fields.
		 *
		 * @since 1.0.0
		 *
		 * @param string $result     HTML for fields.
		 * @param array  $form_data  Form data.
		 * @param int    $approval_id Approval ID.
		 */
		return apply_filters( 'gcforms_approvals_form_builder_get_approval_fields', $result, $form_data, $approval_id );
	}

	/**
	 * Retrieve string of the class names.
	 *
	 * @since 1.0.0
	 *
	 * @param array $classes Array of class names for element.
	 *
	 * @return string
	 */
	protected function get_html_class( $classes ) {

		if ( ! is_array( $classes ) ) {
			$classes = (array) $classes;
		}

		if ( ! gcforms_approvals()->settings->get_prop( 'is_enabled' ) ) {
			$classes[] = 'hidden';
		}

		$classes = array_unique( array_map( 'esc_attr', $classes ) );

		return implode( ' ', $classes );
	}

	/**
	 * Add a new item `Approvals` to panel sidebar.
	 *
	 * @since 1.0.0
	 *
	 * @param array $sections  Registered sections.
	 * @param array $form_data Contains array of the form data (post_content).
	 *
	 * @return array
	 */
	public function panel_sidebar( $sections, $form_data ) {

		$sections['approvals'] = esc_html__( '审批', 'gcforms-approvals' );

		return $sections;
	}

	/**
	 * Save allowed field types to property.
	 *
	 * @since 1.0.0
	 *
	 * @param array $allowed_field_types White list of field types to allow.
	 *
	 * @return array
	 */
	public function form_fields_allowed( $allowed_field_types ) {

		$this->allowed_field_types = ! empty( $allowed_field_types ) ? $allowed_field_types : [ 'all-fields' ];

		return $allowed_field_types;
	}

	/**
	 * Add own localized strings to the Builder.
	 *
	 * @since 1.0.0
	 *
	 * @param array  $strings Localized strings.
	 * @param object $form    Current form.
	 *
	 * @return array
	 */
	public function builder_strings( $strings, $form ) {

		$strings['approval_prompt']        = esc_html__( '输入审批名称', 'gcforms-approvals' );
		$strings['approval_ph']            = '';
		$strings['approval_error']         = esc_html__( '您必须提供审批名称', 'gcforms-approvals' );
		$strings['approval_error2']        = esc_html__( '要禁用所有审批，请使用“审批”下拉设置。', 'gcforms-approvals' );
		$strings['approval_delete']        = esc_html__( '是否确实要删除此审批？', 'gcforms-approvals' );
		$strings['approval_def_name']      = esc_html__( '未命名的审批', 'gcforms-approvals' );
		$strings['approval_required_flds'] = esc_html__( '您的表单包含尚未配置的必需审批设置。请仔细检查并配置这些设置以完成连接设置。', 'gcforms-approvals' );

		return $strings;
	}

	/**
	 * 嵌入一个JavaScript文件和CSS样式文件
	 *
	 * @since 1.0.0
	 */
	public function enqueue_assets() {

		$min = gcforms_get_min_suffix();

		gc_enqueue_script(
			'gcforms-approvals-admin-builder',
			GCFORMS_APPROVALS_URL . "assets/js/approvals-builder.min.js",
			[ 'gcforms-builder' ],
			GCFORMS_APPROVALS_VERSION,
			true
		);

		gc_enqueue_style(
			'gcforms-approvals-admin-builder',
			GCFORMS_APPROVALS_URL . "assets/css/approvals.min.css",
			[ 'gcforms-builder' ],
			GCFORMS_APPROVALS_VERSION
		);

	}
}
