<?php

namespace GCFormsApprovals\Admin;

use GCFormsApprovals\Approval;
use GCFormsApprovals\Helpers\Formatting;

/**
 * Settings class.
 *
 * @since 1.0.0
 */
class Settings {

	/**
	 * Form data.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	private $form_data = [];

	/**
	 * Approvals settings.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	private $approvals = [];

	/**
	 * Is Approvals enabled?
	 *
	 * @since 1.0.0
	 *
	 * @var boolean
	 */
	private $is_enabled = false;

	/**
	 * Initialize.
	 *
	 * @since 1.0.0
	 */
	public function init() {

		if ( empty( $this->approvals ) ) {
			$this->approvals[1] = ( new Approval() )->get_defaults();
		}

		$this->hooks();
	}

	/**
	 * Hooks.
	 *
	 * @since 1.0.0
	 */
	protected function hooks() {

		add_filter( 'gcforms_save_form_args', [ $this, 'save_form_args' ], 11, 3 );
	}

	/**
	 * 编辑表单时，先预处理审批流数据，然后将其保存在form_data中
	 *
	 * @since 1.0.0
	 *
	 * @param array $form Form array, usable with gc_update_post.
	 * @param array $data Data retrieved from $_POST and processed.
	 * @param array $args Empty by default, may have custom data not intended to be saved, but used for processing.
	 *
	 * @return array
	 */
	public function save_form_args( $form, $data, $args ) {

		// Bail if form has no approvals.
		if ( empty( $data['settings']['approvals'] ) ) {
			return $form;
		}

		// Get a filtered form content.
		$form_data = json_decode( stripslashes( $form['post_content'] ), true );

		// Modify content as we need, done by reference.
		foreach ( $form_data['settings']['approvals'] as $approval_id => &$approval_data ) {
			// 如果审批人没填写，删除此条审批流
			if (
				empty( $approval_data['user_id'] ) 
			) {
				unset( $form_data['settings']['approvals'][ $approval_id ] );
				continue;
			}

			$approval_data['id'] = $approval_id;
			$approval            = new Approval( $approval_data, true );

			$approval_data = $approval->get_data();
		}
		unset( $approval_data );

		// Save the modified version back to form.
		$form['post_content'] = gcforms_encode( $form_data );

		return $form;
	}

	/**
	 * Set a collection of props.
	 *
	 * @since 1.0.0
	 *
	 * @param \GCForms_Builder_Panel_Settings $panel_settings GCForms_Builder_Panel_Settings object.
	 */
	public function set_props( $panel_settings ) {

		if ( ! ( $panel_settings instanceof \GCForms_Builder_Panel_Settings ) ) {
			return;
		}

		$this->form_data = $panel_settings->form_data;

		if (
			empty( $this->form_data['settings'] ) ||
			! is_array( $this->form_data['settings'] )
		) {
			return;
		}

		$form_settings = $this->form_data['settings'];

		if ( isset( $form_settings['approvals_enable'] ) ) {
			$this->is_enabled = gc_validate_boolean( $form_settings['approvals_enable'] );
		}

		if ( ! empty( $form_settings['approvals'] ) ) {
			$this->approvals = $form_settings['approvals'];
		}
	}

	/**
	 * Gets a prop for a getter method.
	 *
	 * @since 1.0.0
	 *
	 * @param string $name Name of prop to get.
	 *
	 * @return mixed
	 */
	public function get_prop( $name ) {

		return isset( $this->{$name} ) ? $this->{$name} : null;
	}
}
