<?php

namespace GCFormsApprovals;

use GCForms\Helpers\Crypto;
use GCFormsApprovals\Helpers\Formatting;

/**
 * Class Approval.
 *
 * @since 1.0.0
 */
class Approval {

	/**
	 * 审批流数据
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	protected $data = [];

	/**
	 * 对自定义请求值应用加密
	 *
	 * @since 1.0.0
	 *
	 * @var bool
	 */
	protected $use_encrypt = false;

	/**
	 * 构造器
	 *
	 * @since 1.0.0
	 *
	 * @param array $data        审批流数据
	 * @param bool  $use_encrypt 如果要加密自定义请求值，请传递“true”
	 */
	public function __construct( $data = [], $use_encrypt = false ) {

		$this->use_encrypt = gc_validate_boolean( $use_encrypt );

		if ( ! empty( $data ) && is_array( $data ) ) {
			$this->set_data( $data );
		}
	}

	/**
	 * 设置并净化数据
	 *
	 * @since 1.0.0
	 *
	 * @param array $raw_data 审批流数据
	 */
	protected function set_data( $raw_data ) {

		$defaults = $this->get_defaults();
		$data     = gc_parse_args( $raw_data, $defaults );
		$orders  = gcforms_approvals()->get_available_orders();

		$data['id']      = absint( $data['id'] );
		$data['name']    = sanitize_text_field( $data['name'] );
		$data['user_id']  = absint( $data['user_id'] );
		$data['order']  = isset( $orders[ $data['order'] ] ) ? $data['order'] : $defaults['order'];
		$data['status']    = $this->set_request_params( $data['status'] );

		$this->data = $data;
	}

	/**
	 * 返回此对象的所有数据
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function get_data() {

		return $this->data;
	}

	/**
	 * 设置并净化标头、正文等的请求参数
	 *
	 * @since 1.0.0
	 *
	 * @param array  $raw_params Request parameters.
	 * @param string $context    Context for request parameters - header or body.
	 *
	 * @return array
	 */
	protected function set_request_params( $raw_params, $context = 'header' ) {

		$params = [];

		foreach ( $raw_params as $name => $value ) {

			$name = Formatting::sanitize_status_name( $name );

			// 如果name为空字符串，则跳过params。
			if ( gcforms_is_empty_string( $name ) ) {
				continue;
			}

			if ( false === $value ) {
				continue;
			}

			$params[ $name ] = $value;
		}

		return $params;
	}

	/**
	 * 设置一个默认的数据集
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function get_defaults() {

		return [
			'name'    => esc_html__( '未命名的审批', 'gcforms-approvals' ),
			// 'method'  => 'get',
			// 'format'  => 'json',
			// 'url'     => '',
			'user_id'  => '',
			'order' => 1,
			'status'    => [],
		];
	}
}
