<?php

namespace GCFormsApprovals;

/**
 * 审批流插件类
 *
 * @since 1.0.0
 */
final class Plugin {

	/**
	 * FormBuilder instance.
	 *
	 * @since 1.0.0
	 *
	 * @var \GCFormsApprovals\Admin\FormBuilder
	 */
	public $form_builder;

	/**
	 * Settings instance.
	 *
	 * @since 1.0.0
	 *
	 * @var \GCFormsApprovals\Admin\Settings
	 */
	public $settings;

	/**
	 * Process instance.
	 *
	 * @since 1.0.0
	 *
	 * @var \GCFormsApprovals\Process
	 */
	public $process;

	/**
	 * Constructor method.
	 *
	 * @since 1.0.0
	 */
	private function __construct() {}

	/**
	 * Returns the instance.
	 *
	 * @since  1.0.0
	 *
	 * @return \GCFormsApprovals\Plugin
	 */
	public static function get_instance() {

		static $instance = null;

		if (
			null === $instance ||
			! $instance instanceof self
		) {
			$instance = ( new self() )->init();
		}

		return $instance;
	}

	/**
	 * 所有实际的插件加载都在这里完成
	 *
	 * @since 1.0.0
	 */
	public function init() {

		$this->hooks();

		return $this;
	}

	/**
	 * Hooks.
	 *
	 * @since 1.0.0
	 */
	protected function hooks() {

		add_action( 'gcforms_loaded', [ $this, 'init_components' ] );
		add_action( 'gcforms_updater', [ $this, 'updater' ] );
		add_filter( 'gcforms_helpers_templates_include_html_located', [ $this, 'templates' ], 10, 4 );
	}

	/**
	 * 初始化组件
	 *
	 * @since 1.0.0
	 */
	public function init_components() {

		if (
			gcforms_is_admin_page( 'builder' ) ||
			gc_doing_ajax()
		) {
			$this->form_builder = new Admin\FormBuilder();
			$this->settings     = new Admin\Settings();

			$this->form_builder->init();
			$this->settings->init();
		}

		$this->process = new Process();
		$this->process->init();
	}

	/**
	 * Load the addon updater.
	 *
	 * @since 1.0.0
	 *
	 * @param string $key License key.
	 */
	public function updater( $key ) {

		new \GCForms_Updater(
			[
				'plugin_name' => 'GCForms Approvals',
				'plugin_slug' => 'gcforms-approvals',
				'plugin_path' => plugin_basename( GCFORMS_APPROVALS_FILE ),
				'plugin_url'  => trailingslashit( GCFORMS_APPROVALS_URL ),
				'version'     => GCFORMS_APPROVALS_VERSION,
				'key'         => $key,
			]
		);
	}

	/**
	 * Change a template location.
	 *
	 * @since 1.0.0
	 *
	 * @param string $located  Template location.
	 * @param string $template Template.
	 * @param array  $args     Arguments.
	 * @param bool   $extract  Extract arguments.
	 *
	 * @return string
	 */
	public function templates( $located, $template, $args, $extract ) {

		// Checking if `$template` is an absolute path and passed from this plugin.
		if (
			( 0 === strpos( $template, GCFORMS_APPROVALS_PATH ) ) &&
			is_readable( $template )
		) {
			return $template;
		}

		return $located;
	}

	/**
	 * 审批人列表
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function get_approver_users() {
		$users      = get_users( [ 'fields' => [ 'ID', 'display_name' ] ] );
		$users      = gc_list_pluck( $users, 'display_name', 'ID' );
		return $users;
	}

	/**
	 * 审批流程，默认10个环节
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function get_available_orders() {

		return [
			'1' => esc_html__( 'Lv1 第一审批人', 'gcforms-approvals' ),
			'2' => esc_html__( 'Lv2', 'gcforms-approvals' ),
			'3' => esc_html__( 'Lv3', 'gcforms-approvals' ),
			'4' => esc_html__( 'Lv4', 'gcforms-approvals' ),
			'5' => esc_html__( 'Lv5', 'gcforms-approvals' ),
			'6' => esc_html__( 'Lv6', 'gcforms-approvals' ),
			'7' => esc_html__( 'Lv7', 'gcforms-approvals' ),
			'8' => esc_html__( 'Lv8', 'gcforms-approvals' ),
			'9' => esc_html__( 'Lv9', 'gcforms-approvals' ),
			'10' => esc_html__( 'Lv10 最终审批人', 'gcforms-approvals' ),
		];
	}

	/**
	 * 审批意见，审批状态、审批结果
	 * 	 
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function get_available_status() {

		return [
			'confirm' => esc_html__( '通过', 'gcforms-approvals' ),
			'reject' => esc_html__( '驳回', 'gcforms-approvals' ),
		];
	}

	/**
	 * get_available_status 与自定义的审批状态进行组合
	 * 
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function get_available_custom_status($custom_status){
		$status = $this->get_available_status();
		if(empty($custom_status)){
			return $status;
		}
		foreach ($custom_status as $key => $custom) {
			$status[$key] = $custom;
		}
		return $status;
	}

}
