<?php

namespace GCFormsApprovals;

use GCForms\Tasks\Meta;
use GCForms\Helpers\Crypto;
use GCFormsApprovals\Helpers\Formatting;

/**
 * Class Process.
 *
 * @since 1.0.0
 */
class Process {

	/**
	 * Action name for async task.
	 *
	 * @since 1.0.0
	 */
	const ACTION = 'gcforms_approvals_process_delivery_approval';

	public $alerts = [];

	/**
	 * Array of form fields.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	protected $fields = [];

	/**
	 * Submitted form content.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	protected $entry = [];

	/**
	 * Form data and settings.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	protected $form_data = [];

	/**
	 * 申请单ID
	 *
	 * @since 1.0.0
	 *
	 * @var int
	 */
	protected $entry_id;

	/**
	 * Initialize.
	 *
	 * @since 1.0.0
	 */
	public function init() {

		$this->hooks();
	}

	/**
	 * Hooks.
	 *
	 * @since 1.0.0
	 */
	protected function hooks() {
		add_action( 'gcforms_process_complete', [ $this, 'process' ], 10, 4 );  // 提交表单时，添加审批相关动作
		add_action( 'gcforms_entries_table_column_status',  [ $this, 'get_column_status']); // 启用状态列
		add_action( 'gcforms_entries_init', [ $this, 'process_approval_add' ], 8, 1 ); // 存储审批变更
		add_action( 'gcforms_entry_details_content', [ $this, 'details_approval' ], 12, 2 ); // 在详情页添加审批编辑区块
	}

	/**
	 * 用户提交表单时触发.
	 *
	 * @since 1.0.0
	 *
	 * @param array $fields    Array of form fields.
	 * @param array $entry     Submitted form content.
	 * @param array $form_data Form data and settings.
	 * @param int   $entry_id  ID of a saved entry.
	 */
	public function process( $fields, $entry, $form_data, $entry_id ) {
		global $gcdb;

		if (
			empty( $form_data['settings']['approvals'] ) ||
			! isset( $form_data['settings']['approvals_enable'] ) ||
			! gc_validate_boolean( $form_data['settings']['approvals_enable'] )
		) {
			return;
		}

		$this->fields    = $fields;
		$this->entry     = $entry;
		$this->form_data = $form_data;
		$this->entry_id  = $entry_id;

		$gcdb->update( 
			$gcdb->prefix . 'gcforms_entries', 
			array( 'status' => '待审批' ), 
			array( 'entry_id' => $entry_id ),
			array( '%s' ),
			array( '%d' )
		);

		$this->process_entry_meta_set( $entry_id, $entry['id'], '');

	}

	public function get_column_status() {
		return true;
	}


	/**
	 * 提交审批动作，修改条目状态并添加日志
	 *
	 * @todo Convert to AJAX
	 *
	 * @since 1.1.6
	 */
	public function process_approval_add() {

		// 检查触发后和所需变量
		if ( empty( $_POST['gcforms_add_approval'] ) || empty( $_POST['entry_id'] ) || empty( $_POST['form_id'] ) || empty( $_POST['entry_approval'] ) ) {
			return;
		}

		// 安全检查
		if ( empty( $_POST['_gcnonce'] ) || ! gc_verify_nonce( sanitize_key( $_POST['_gcnonce'] ), 'gcforms_entry_details_addapproval' ) ) {
			return;
		}

		$entry_id   = absint( $_POST['entry_id'] );
		$form_id    = absint( $_POST['form_id'] );
		$approval_status = $_POST['entry_approval'];

		$this->process_entry_meta_set( $entry_id, $form_id, $approval_status);
		
		\GCForms\Admin\Notice::add( '审批意见提交成功！', 'success' );
		
	}

	/**
	 * $entry_id 申请单ID
	 * $form_id  表单ID
	 * $approval_status 选择的审批意见
	 */
	public function process_entry_meta_set($entry_id, $form_id='', $status=''){
		global $gcdb;
		if(empty($form_id)){
			$entry =  gcforms()->entry->get($entry_id);
			$form_id = $entry->form_id;
		}

		// -----  01 处理待审核记录  -----
		$approval_pending = gcforms()->entry_meta->get_meta(
			[
				'entry_id' => $entry_id,
				'type'     => 'approval_pending'
			]
		);

		if ( isset( $approval_pending[0] ) ) {
			// 获取状态集合
			$status_list = gcforms_approvals()->get_available_custom_status( $approval_pending[0]->status );
			gcforms()->entry_meta->update(
				$approval_pending[0]->id,
				[
					'type'     => 'approval_log',
					'status' => $status_list[$status],
				]
			);

			if( $status == 'reject' ){
				$gcdb->update( 
					$gcdb->prefix . 'gcforms_entries', 
					array( 'status' => '已驳回' ), 
					array( 'entry_id' => $entry_id ),
					array( '%s' ),
					array( '%d' )
				);
				return;
			}else{
				$gcdb->update( 
					$gcdb->prefix . 'gcforms_entries', 
					array( 'status' => '审批中' ), 
					array( 'entry_id' => $entry_id ),
					array( '%s' ),
					array( '%d' )
				);
			}
		}

		// ----- 02 开启下一个审批环节 -----
		$approvals = $this->process_approvals_and_metas( $entry_id, $form_id );
		
		$approvals_complete = 0; // 用来判断审批流程是否已经完成
		foreach( $approvals as $id=>$approval ){
			if( isset($approval['meta']) ){
				$approvals_complete++;
				continue;
			}
			//判断自动逻辑
			if( 'confirm' == $this->is_conditionals_passed( $approval ) ){
				$approvals_complete++;
				// 自动审批通过
				gcforms()->entry_meta->add(
					[
						'entry_id' => $entry_id,
						'form_id'  => $form_id,
						'user_id'  => $approval['user_id'],
						'type'     => 'approval_log',
						'status' => '自动通过',
						'data'     => json_encode($approval),
					],
					'entry_meta'
				);
				$gcdb->update( 
					$gcdb->prefix . 'gcforms_entries', 
					array( 'status' => '审批中' ), 
					array( 'entry_id' => $entry_id ),
					array( '%s' ),
					array( '%d' )
				);
			}elseif( 'reject' == $this->is_conditionals_passed( $approval ) ){
				// 自动审批驳回
				gcforms()->entry_meta->add(
					[
						'entry_id' => $entry_id,
						'form_id'  => $form_id,
						'user_id'  => $approval['user_id'],
						'type'     => 'approval_log',
						'status' => '自动驳回',
						'data'     => json_encode($approval),
					],
					'entry_meta'
				);
				$gcdb->update( 
					$gcdb->prefix . 'gcforms_entries', 
					array( 'status' => '已驳回' ), 
					array( 'entry_id' => $entry_id ),
					array( '%s' ),
					array( '%d' )
				);
				break;
			}else{
				// 标记待审批人
				gcforms()->entry_meta->add(
					[
						'entry_id' => $entry_id,
						'form_id'  => $form_id,
						'user_id'  => $approval['user_id'],
						'type'     => 'approval_pending',
						'data'     => json_encode($approval),
					],
					'entry_meta'
				);
				break;
			}

		}

		// 审批流程全部结束，审核通过
		if( $approvals_complete == count($approvals) ){
			$gcdb->update( 
				$gcdb->prefix . 'gcforms_entries', 
				array( 'status' => '已通过' ), 
				array( 'entry_id' => $entry_id ),
				array( '%s' ),
				array( '%d' )
			);
		}
	}

	/**
	 * 后台详情页添加审批动作
	 */
	public function details_approval($entry, $form_data){

		// 先判断当前登录用户是否属于当前审批状态
		$user_ID = get_currentuserinfo()->ID;

		$action_url = add_query_arg(
			[
				'page'     => 'gcforms-entries',
				'view'     => 'details',
				'entry_id' => absint( $entry->entry_id ),
			],
			admin_url( 'admin.php' )
		);
		$form_id    = ! empty( $form_data['id'] ) ? $form_data['id'] : $entry->form_id;
		?>
		<!-- Entry Notes metabox -->
		<style>
			#gcforms-entries-single #gcforms-entry-approvals .inside {
			  margin: 0;
			  padding: 0;
			}

			#gcforms-entries-single #gcforms-entry-approvals .btns {
			  overflow: auto;
			  padding: 8px 0;
			}

			#gcforms-entries-single #gcforms-entry-approvals .no-approvals {
			  margin: 0;
			  padding: 4px 10px 8px;
			}

			#gcforms-entries-single #gcforms-entry-approvals .gcforms-entry-approvals-single {
			  padding: 10px;
			  border-top: 1px solid #e3e1e3;
			}

			#gcforms-entries-single #gcforms-entry-approvals .gcforms-entry-approvals-single:first-of-type {
			  border-top: 0;
			}

			#gcforms-entries-single #gcforms-entry-approvals .gcforms-entry-approvals-single p:last-of-type {
			  margin-bottom: 0;
			}

			#gcforms-entries-single #gcforms-entry-approvals .gcforms-entry-approvals-single.even {
			  background-color: #f9f9f9;
			}

			#gcforms-entries-single #gcforms-entry-approvals .gcforms-entry-approvals-byline {
			  color: #888;
			  margin-bottom: 10px;
			}

			#gcforms-entries-single #gcforms-entry-approvals .gcforms-entry-approvals-byline .notes-user {
			  text-decoration: none;
			  font-weight: 600;
			}
		</style>
		<div id="gcforms-entry-approvals" class="postbox">

			<div class="postbox-header">
				<h2 class="hndle">
					<span><?php esc_html_e( '审批', 'gcforms' ); ?></span>
				</h2>
			</div>

			<div class="inside">
			<?php 
			$entry_approvals = $this->process_approvals_and_metas($entry->entry_id, $form_id,$form_data);
			$count = 1;
			$orders = gcforms_approvals()->get_available_orders();
			foreach( $entry_approvals as $id => $approval ){ 
				$user = get_user_by( 'id' , $approval['user_id'] );
				$class       = 0 === $count % 2 ? 'even' : 'odd';
				$approval_meta_data = json_decode($approval['meta']->data);
				echo '<div class="gcforms-entry-approvals-single '. $class .'">';
				echo '<p>审批流：'. $approval['name'] .'</p>';
				echo '<p>审批排序：'. $orders[$approval['order']] .'</p>';
				echo '<p>审批人：'. $user->user_nicename .' ( '. $user->user_email .' )</p>';
		
				if( isset($approval['meta']) ){
					echo '<p>审批意见：'. $approval['meta']->status .'</p>';
					echo '<p>审批时间：'. $approval['meta']->date .'</p>';
				}elseif( $approval['type'] == 'approval_pending' && $approval['user_id'] == $user_ID && gcforms_current_user_can( 'edit_entries_form_single', $form_id && $approval_meta_data['id'] = $approval['id'] ) ){
			?>
					<p>

						<form action="<?php echo esc_url( $action_url ); ?>" method="post">
							审批意见：
							<select  name="entry_approval">
								<?php 
									$status = gcforms_approvals()->get_available_custom_status($approval['status']);
									foreach( $status as $key => $value ){
										echo '<option value ="'.$key.'">'.$value.'</option>';
									}
								?>
							</select>
							<?php gc_nonce_field( 'gcforms_entry_details_addapproval' ); ?>
							<input type="hidden" name="entry_id" value="<?php echo absint( $entry->entry_id ); ?>">
							<input type="hidden" name="form_id" value="<?php echo absint( $form_id ); ?>">
							<input type="submit" name="gcforms_add_approval" class="save button-primary" value="<?php esc_attr_e( '确认审批', 'gcforms' ); ?>">
						</form>

					</p>
			<?php
				}else{
					echo '<p>审批意见：</p>';
					echo '<p>审批时间：</p>';
				}
				echo '</div>';
				$count++;
			}
			?>

			</div>

		</div>
		<?php

	}

	/** 审批流程与审批记录混合形成流程单
	 * $approvals 自定义的审批流
	 */
	public function process_approvals_and_metas($entry_id, $form_id='',$form_data=''){
		if(empty($form_id)){
			$entry =  gcforms()->entry->get($entry_id);
			$form_id = $entry->form_id;
		}

		//获取审批流数据集合
		if(empty($form_data)){
			$form_data = gcforms()->form->get( $form_id, [ 'content_only' => true ] );
		}

		// 后台定义的审批流程
		$approvals = $form_data['settings']['approvals'];

		usort($approvals, [ $this, 'field_order' ]);

		// 当前待审核
		$approval_pending = gcforms()->entry_meta->get_meta(
			[
				'entry_id' => $entry_id,
				'type'     => 'approval_pending'
			]
		);
		$approval_pending_data = isset( $approval_pending[0] ) ? json_decode( $approval_pending[0]->data ) : '';

		// 全部审批记录
		$entry_meta_approvals = gcforms()->entry_meta->get_meta(
			[
				'entry_id' => $entry_id,
				'type'     => 'approval_log',
				'orderby'        => 'id',
				'order'          => 'ASC',
			]
		);

		foreach($approvals as $id=>$approval){
			// 给审批流标记待审核的字段
			if ( $approval_pending_data->id == $approval['id'] ) {
				$approvals[$id]['type'] = 'approval_pending';
			}

			$approval_user = get_user_by('ID', $approval['user_id']);
			if(!isset($approval_user)){
				continue;
			}
			foreach($entry_meta_approvals as $meta){
				if(json_decode($meta->data)->id == $approval['id']){
					$approvals[$id]['meta'] = $meta;
				}
			}
			
		}

		return $approvals;
	}

	public function field_order( $a, $b ) {

		return $a['order'] - $b['order'];
	}

	

	/**
	 * 处理审批的条件逻辑。
	 *
	 * @since 1.0.0
	 *
	 * @param array $approval 审批流数据
	 *
	 * @return true为通过，false为驳回, null为不处理
	 */
	protected function is_conditionals_passed( $approval ) {

		if (
			empty( $approval['conditional_logic'] ) ||
			empty( $approval['conditionals'] )
		) {
			return;
		}

		// 是否符合逻辑判断条件
		$pass = gcforms_conditional_logic()->process( $this->fields, $this->form_data, $approval['conditionals'] );
		if(!$pass){
			return;
		}

		// 判断是否自动通过
		if (
			! empty( $approval['conditional_type'] ) &&
			'confirm' === $approval['conditional_type']
		) {
			return 'confirm';
		}

		return 'reject';
	}

}
